import datetime


def handle_datetime(agefield):
    if agefield is None:
        agefield = None
    else:
        if isinstance(agefield, datetime.datetime):
            agefield = agefield.strftime("%Y-%m-%d %H:%M:%S")
        else:
            agefield = '解析失败'
    return agefield


def handle_every_trigger(trigger):
    # print(f"handle_before： {trigger}")
    if 'timezone' in trigger:
        timezone = trigger.get('timezone')
        if timezone is None:
            timezone = None
        else:
            timezone = timezone.__str__()
        trigger['timezone'] = timezone
    if 'start_date' in trigger:
        trigger['start_date'] = handle_datetime(agefield=trigger.get('start_date'))
    if 'end_date' in trigger:
        trigger['end_date'] = handle_datetime(agefield=trigger.get('end_date'))
    if 'interval' in trigger:
        interval = trigger.get('interval')
        if interval is None:
            interval = None
        else:
            interval = interval.total_seconds()
        trigger['interval'] = interval
    # print(f"handle_after： {trigger}")
    return trigger


def handle_trigger(trigger):
    if isinstance(trigger, list):
        temp = []
        for every_trigger in trigger:
            temp.append(handle_every_trigger(every_trigger))
        return temp
    else:
        return handle_every_trigger(trigger)



import os
import json
import six
import traceback
from typing import Dict

from uhashring import HashRing
from apscheduler.jobstores.redis import RedisJobStore
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.util import ref_to_obj

from fastapi_crawler_scheduler.service.dbRedisHelper import DbRedisHelper, standard_time


class BaseScheduler(object):
    """
    任务管理和自身的异步任务
    :param DbRedisHelper对象 redis_db: DbRedisHelper对象
    :param BackgroundScheduler对象 scheduler: BackgroundScheduler对象
    :param str uuid_number: uuid
    :param str project_name: 项目名字，主要区分不同项目
    :param RedisJobStore对象 redis_job_store: RedisJobStore对象
    """

    def __init__(
            self,
            redis_db: DbRedisHelper,
            scheduler: BackgroundScheduler,
            uuid_number: str,
            project_name: str,
            redis_job_store: RedisJobStore,

    ) -> None:
        self.uuid_number = uuid_number
        self.project_name = project_name
        self.redis_db = redis_db
        self.scheduler = scheduler
        self.process_id_list = []
        self.redis_job_store = redis_job_store

    def scheduler_add_job(
            self,
            **task_info
    ) -> None:
        '''
        向 apscheduler 中添加一个任务
        :param crawler_info: 任务参数
        :return:
        '''
        try:
            func = ref_to_obj(task_info.get('func'))
        except Exception as e:
            print(f"scheduler_add_job   函数错误 ：{e}")
            print(traceback.format_exc())
            return
        trigger = task_info.get('trigger')
        job_id = task_info.get('job_id')
        crawler_info = task_info.get('crawler_info')
        trigger_args = task_info.get('trigger_args')
        if self.redis_job_store.redis.hexists(self.redis_job_store.jobs_key, job_id):
            self.redis_job_store.remove_job(job_id=job_id)
        self.scheduler.add_job(
            func=func,
            id=job_id,
            trigger=trigger,
            kwargs=crawler_info,
            **trigger_args,
        )

    def check_process(self) -> None:
        '''
        检查进程
        :return:
        '''
        self.redis_db.process_acquire(f'{self.project_name}:node:{self.uuid_number}:{os.getpid()}')

    def get_process_list(self) -> list:
        '''
        获取当前项目所有进程信息
        :return:
        '''
        return self.redis_db.get_proces_info()

    def process_check_count(self, process_id: int, check_key: str) -> None:
        '''
        检查任务策略
        :param process_id: 进程pid
        :param check_key:  待检查任务键
        :return:
        '''
        if process_id not in self.process_id_list:
            if self.redis_db.acquire(lock_name=check_key):
                check_value = self.redis_db.from_key_get_value(key_name=check_key)
                if check_value is None:
                    return None
                check_process_number = check_value.get('check_process_number')
                if check_process_number is None:
                    check_process_number = 1
                else:
                    check_process_number += 1
                if check_process_number >= 2:
                    self.redis_db.delete_key(lock_name=check_key)
                else:
                    check_value['check_process_number'] = check_process_number
                    self.redis_db.string_set(key=check_key, value=json.dumps(check_value, ensure_ascii=False))
                self.redis_db.release(lock_name=check_key)

    def check_backend_task(self) -> None:
        '''
        检查后端任务
        :return:
        '''
        node_process_id_list = self.get_process_list()
        self.process_id_list = [int(str(node_id).strip().split(':')[-1]) for node_id in node_process_id_list]
        # 处理后端操作
        hr = HashRing(nodes=node_process_id_list)
        for backend_key in self.redis_db.get_backend_task():
            backend_info = self.redis_db.from_key_get_value(backend_key)
            if backend_info is None:
                continue
            lock_all_backend_key = f'{self.redis_db.prefix}:all:{backend_info["job_id"]}'
            all_backend_key = f'{self.project_name}:all:{backend_info["job_id"]}'
            if not self.redis_db.lock_exists(lock_name=lock_all_backend_key):
                process_node_id = hr.get_node(backend_info["job_id"])
                backend_info['process_node_id'] = process_node_id
                self.redis_db.string_set(key=all_backend_key, value=json.dumps(backend_info, ensure_ascii=False))
                self.redis_db.delete_key(lock_name=backend_key)

    def check_all_task(self) -> None:
        '''
        检查所有任务
        :return:
        '''
        node_process_id_list = self.get_process_list()
        self.process_id_list = [int(str(node_id).strip().split(':')[-1]) for node_id in node_process_id_list]
        hr = HashRing(nodes=node_process_id_list)
        # 处理 all_task
        for all_key in self.redis_db.get_all_task():
            all_value = self.redis_db.from_key_get_value(all_key)
            if all_value is None:
                continue
            if self.redis_db.acquire(lock_name=all_key):
                redis_process_node_id = all_value["process_node_id"]
                new_process_node_id = hr.get_node(all_value["job_id"])
                if new_process_node_id == redis_process_node_id:
                    process_id = int(str(redis_process_node_id).strip().split(':')[-1])
                    # 进程没变化
                    if all_value['is_change'] == 1:
                        next_key = f'{self.project_name}:{all_value["operation"]}:{self.uuid_number}:{all_value["job_id"]}:{process_id}'
                        self.redis_db.string_set(key=next_key, value=json.dumps(all_value, ensure_ascii=False))
                        all_value["is_change"] = 0
                        self.redis_db.string_set(key=all_key, value=json.dumps(all_value, ensure_ascii=False))
                    else:
                        pass
                else:
                    # 进程有变化
                    new_process_id = int(str(new_process_node_id).strip().split(':')[-1])
                    redis_process_id = int(str(redis_process_node_id).strip().split(':')[-1])
                    delete_key = f'{self.project_name}:delete:{self.uuid_number}:{all_value["job_id"]}:{redis_process_id}'
                    all_value["details"] = "删除任务"
                    self.redis_db.string_set(key=delete_key, value=json.dumps(all_value, ensure_ascii=False))
                    all_value["process_node_id"] = new_process_node_id
                    if all_value['operation'] == 'delete':
                        # 无需向下一步添加任务
                        pass
                    else:
                        insert_key = f'{self.project_name}:insert:{self.uuid_number}:{all_value["job_id"]}:{new_process_id}'
                        all_value["details"] = "进程变化，新增任务"
                        self.redis_db.string_set(key=insert_key, value=json.dumps(all_value, ensure_ascii=False))
                    all_value["is_change"] = 0
                    self.redis_db.string_set(key=all_key, value=json.dumps(all_value, ensure_ascii=False))
                self.redis_db.release(lock_name=all_key)

    def check_insert_task(self) -> None:
        '''
        检查新增任务
        :return:
        '''
        for insert_key in self.redis_db.get_insert_task():
            insert_value = self.redis_db.from_key_get_value(insert_key)
            if insert_value is None:
                continue
            try:
                process_node_id = insert_value['process_node_id']
                apscheduler_id = insert_value['job_id']
                process_id = int(str(process_node_id).strip().split(':')[-1])
                if os.getpid() == process_id:
                    if self.scheduler.get_job(job_id=apscheduler_id):
                        self.scheduler.remove_job(job_id=apscheduler_id)
                    self.scheduler_add_job(**insert_value)
                    self.redis_db.delete_key(lock_name=insert_key)
                else:
                    pass
                    self.process_check_count(process_id=process_id, check_key=insert_key)
            except Exception as e:
                print(f"check_insert_task 函数 错误 ：{e}")
                print(traceback.format_exc())

    def check_update_task(self) -> None:
        '''
        检查更新任务
        :return:
        '''
        for update_key in self.redis_db.get_update_task():
            update_value = self.redis_db.from_key_get_value(update_key)
            if update_value is None:
                continue
            try:
                process_node_id = update_value['process_node_id']
                apscheduler_id = update_value['job_id']
                process_id = int(str(process_node_id).strip().split(':')[-1])
                if os.getpid() == process_id:
                    if self.scheduler.get_job(job_id=apscheduler_id):
                        self.scheduler.remove_job(job_id=apscheduler_id)
                    self.scheduler_add_job(**update_value)
                    self.redis_db.delete_key(lock_name=update_key)
                else:
                    self.process_check_count(process_id=process_id, check_key=update_key)
            except Exception as e:
                print(f"check_update_task 函数 错误 ：{e}")
                print(traceback.format_exc())

    def check_delete_task(self) -> None:
        '''
        检查删除任务
        :return:
        '''

        for delete_key in self.redis_db.get_delete_task():
            delete_value = self.redis_db.from_key_get_value(delete_key)
            if delete_value is None:
                continue
            try:
                process_node_id = delete_value['process_node_id']
                apscheduler_id = delete_value['job_id']
                process_id = int(str(process_node_id).strip().split(':')[-1])
                if os.getpid() == process_id:
                    if self.scheduler.get_job(job_id=apscheduler_id):
                        self.scheduler.remove_job(job_id=apscheduler_id)
                    self.redis_db.delete_key(lock_name=delete_key)
                    self.redis_db.delete_key(
                        lock_name=f'{self.project_name}:running_job:{os.getpid()}:{apscheduler_id}')
                else:
                    self.process_check_count(process_id=process_id, check_key=delete_key)
            except Exception as e:
                print(f"check_delete_task 函数 错误 ：{e}")
                print(traceback.format_exc())

    def insert_task(self, task_info: Dict) -> None:
        '''
        新增任务
        :param task_info:
        :return:
        '''
        task_info['operation'] = 'insert'
        task_info['is_change'] = 1
        self.redis_db.string_set(f'{self.project_name}:backend:{task_info.get("job_id")}',
                                 json.dumps(task_info, ensure_ascii=False))

    def delete_task(self, job_id: str) -> None:
        '''
        删除任务
        :param job_id:
        :return:
        '''
        crawler_info = dict()
        crawler_info['job_id'] = job_id
        crawler_info['is_change'] = 1
        crawler_info['operation'] = 'delete'
        self.redis_db.string_set(f'{self.project_name}:backend:{crawler_info.get("job_id")}',
                                 json.dumps(crawler_info, ensure_ascii=False))

    def sync_running_job(self) -> None:
        '''
        同步当前apscheduler进程中的任务到redis
        :return:
        '''
        job_list = self.scheduler.get_jobs()
        for job in job_list:
            key = f'{self.project_name}:running_job:{os.getpid()}:{job.id}'
            lock_value = standard_time()
            self.redis_db.process_acquire(lock_name=key, lock_value=lock_value, expire_time=180)

    def check_redis_jobstores(self):
        '''
        异步检查 redis 中 apscheduler 的任务
        :return:
        '''
        job_process_dict = {}
        for all_key in self.redis_db.get_all_task():
            all_value = self.redis_db.from_key_get_value(all_key)
            if all_value is None:
                continue
            process_node_id = all_value["process_node_id"]
            job_id = all_value["job_id"]
            process_id = int(str(process_node_id).strip().split(':')[-1])
            job_process_dict[job_id] = process_id
        if len(job_process_dict) == 0:
            return job_process_dict
        stores_job_task = self.redis_db.get_stores_job_task()
        for stores_job_key in stores_job_task:
            stores_job_run_times_key = stores_job_key.replace('jobs', 'run_times')
            job_states = self.redis_job_store.redis.hgetall(stores_job_key)
            for job_id, job_state in six.iteritems(job_states):
                job_id = job_id.decode('utf-8')
                try:
                    belongs_to_process_id = job_process_dict[job_id]
                except KeyError:
                    continue
                try:
                    stores_job_process_id = int(stores_job_key.split(':')[-1])
                except ValueError:
                    continue
                if stores_job_process_id == belongs_to_process_id:
                    pass
                else:
                    if self.redis_job_store.redis.hexists(stores_job_key, job_id):
                        pass
                        with self.redis_job_store.redis.pipeline() as pipe:
                            pipe.hdel(stores_job_key, job_id)
                            pipe.execute()
                        with self.redis_job_store.redis.pipeline() as pipe:
                            pipe.zrem(stores_job_run_times_key, job_id)
                            pipe.execute()
                    continue

    def run(self) -> None:
        '''
        异步检查所有任务
        :return:
        '''
        self.check_backend_task()
        self.check_all_task()
        self.check_insert_task()
        self.check_update_task()
        self.check_delete_task()
        self.sync_running_job()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_crawler_scheduler',
 'fastapi_crawler_scheduler.service',
 'fastapi_crawler_scheduler.utils']

package_data = \
{'': ['*']}

install_requires = \
['APScheduler>=3.9.1,<4.0.0',
 'fastapi-utils>=0.2.1,<0.3.0',
 'fastapi>=0.85.0,<0.86.0',
 'redis>=4.3.4,<5.0.0',
 'uhashring>=2.1,<3.0']

setup_kwargs = {
    'name': 'fastapi-crawler-scheduler',
    'version': '2.0.7',
    'description': '新增查看任务的方法',
    'long_description': '*********\n\n# fastapi_crawler_scheduler\n\n*********\n\n## 使用\n\n*********\n\n```python\nimport uuid\nfrom fastapi import FastAPI\nfrom fastapi_crawler_scheduler import TaskScheduler\n\napp = FastAPI()\n\ntask_scheduler = TaskScheduler(\n    app=app,\n    ssl=False,\n    project_name="project_name",\n    uuid_number=uuid.uuid4().__str__(),\n    redis_username=\'redis_username\',\n    redis_password=\'redis_password\',\n    redis_host="127.0.0.1",\n    redis_port=6379,\n    thread_pool_size=50,\n)\n```\n\n### 添加|更新任务 - add_task\n\n#### interval类型\n\n```python\ndef add_spider(**crawler_info):\n    print(f"add_spider = {crawler_info}")\n    print("add_spider")\n\n\ntrigger = \'interval\'\ncrawler_info = {\n    "topic": "interval insert_task",\n    "title_handler_name": "interval insert_task",\n    "seconds": 4,\n}\njob_id = \'job_1\'\ntask_scheduler.add_task(\n    func=add_spider,\n    job_id=job_id,\n    trigger=trigger,\n    crawler_info=crawler_info,\n    seconds=4\n)\n```\n\n#### date类型\n\n```python\ndef add_spider(**crawler_info):\n    print(f"add_spider = {crawler_info}")\n    print("add_spider")\n\n\ntrigger = \'date\'\ncrawler_info = {\n    "topic": "date insert_task",\n    "title_handler_name": "date insert_task",\n    "run_date": "2022-10-03 11:30:00",\n}\njob_id = \'job_1\'\nrun_date = \'2022-10-03 11:30:00\'\ntask_scheduler.add_task(\n    func=add_spider,\n    job_id=job_id,\n    trigger=trigger,\n    crawler_info=crawler_info,\n    run_date=run_date,\n)\n```\n\n#### cron类型\n\n```python\ndef add_spider(**crawler_info):\n    print(f"add_spider = {crawler_info}")\n    print("add_spider")\n\n\njob_id = \'job_1\'\ntrigger = \'cron\'\nminute = \'*/2\'\ncrawler_info = {\n    "topic": "cron update_task",\n    "title_handler_name": "cron update_task",\n    "minute": minute,\n}\ntask_scheduler.add_task(\n    func=add_spider,\n    job_id=job_id,\n    trigger=trigger,\n    crawler_info=crawler_info,\n    minute=minute,\n)\n```\n\n### 删除任务 - delete_task\n\n```python\njob_id = \'job_1\'\ntask_scheduler.delete_task(job_id=job_id)\n```\n\n### 查看任务\n\n```python\n# 查看该项目的所有键\ntask_scheduler.show_all_redis_key()\n# 查看该项目的所进程\ntask_scheduler.show_all_redis_nodes()\n# 查看该项目的所有加载过的任务\ntask_scheduler.show_all_redis_tasks()\n# 查看该项目使用的apscheduler.get_jobs()方法获得的所有任务\ntask_scheduler.show_all_scheduler_get_jobs()\n# 查看该项目的apscheduler存储redis的所有任务\ntask_scheduler.show_all_apscheduler_stores_jobs()\n# 查看该项目的的apscheduler存储redis的所有任务的run_ttimes\ntask_scheduler.show_all_redis_apscheduler_run_times()\n# 查看该项目的所有任务不存在apscheduler存储redis的所有任务\ntask_scheduler.show_all_task_not_in_stores_jobs()\n# 查看该项目的任务没有执行的项目（不包含只执行过一次的项目）\ntask_scheduler.show_all_task_not_in_stores_run_times()\n```\n\n安装\n============\nPypi\n----\n\n    $ pip install fastapi-crawler-scheduler\n\n',
    'author': 'laowang',
    'author_email': '847063657@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Callable, Dict
import os

from apscheduler.executors.pool import ThreadPoolExecutor
from apscheduler.jobstores.redis import RedisJobStore
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.jobstores.base import ConflictingIdError, JobLookupError
from apscheduler.util import datetime_to_utc_timestamp
from fastapi import FastAPI
from fastapi_utils.tasks import repeat_every
from apscheduler.util import obj_to_ref

try:
    import cPickle as pickle
except ImportError:  # pragma: nocover
    import pickle

try:
    from redis import Redis
except ImportError:  # pragma: nocover
    raise ImportError('RedisJobStore requires redis installed')

from fastapi_crawler_scheduler.utils.exception import SchedulerError
from fastapi_crawler_scheduler.service.dbRedisHelper import DbRedisHelper
from fastapi_crawler_scheduler.service.baseScheduler import BaseScheduler
from fastapi_crawler_scheduler.utils.objectToString import handle_trigger
from fastapi_crawler_scheduler.utils.objectToString import handle_datetime


class TaskScheduler(object):

    def __init__(self,
                 app: FastAPI,
                 project_name: str,
                 uuid_number: str,
                 ssl: bool,
                 thread_pool_size: int = 10,
                 job_coalesce: bool = True,
                 job_max_instance: int = 1,
                 job_misfire_grace_time: int = 10,
                 redis_host: str = "127.0.0.1",
                 redis_port: int = 6379,
                 redis_username: str = None,
                 redis_password: str = None,
                 ):
        self.app = app
        self.ssl = ssl
        self.project_name = project_name
        self.uuid_number = uuid_number
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.username = redis_username
        self.password = redis_password
        self.jobs_key = self.project_name + ':apscheduler:jobs:' + uuid_number + ":" + str(os.getpid())
        self.run_times_key = self.project_name + ':apscheduler:run_times:' + uuid_number + ":" + str(os.getpid())
        self.thread_pool_size = thread_pool_size
        self.job_coalesce = job_coalesce
        self.job_max_instance = job_max_instance
        self.job_misfire_grace_time = job_misfire_grace_time
        self.redis = Redis(
            host=self.redis_host,
            port=self.redis_port,
            username=self.username,
            password=self.password,
            decode_responses=True,
            ssl=ssl,
        )
        self.pickle_protocol = pickle.HIGHEST_PROTOCOL

        # 实现 scheduler 注册
        self.register_scheduler()
        self.register_redis_db = DbRedisHelper(
            project_name=self.project_name,
            redis_host=self.redis_host,
            redis_port=self.redis_port,
            username=self.username,
            password=self.password,
            ssl=self.ssl,
        )
        self.register_base_scheduler = BaseScheduler(
            project_name=self.project_name,
            redis_db=self.register_redis_db,
            scheduler=self.scheduler,
            uuid_number=self.uuid_number,
            redis_job_store=self.redis_job_store
        )
        self.register_async_task()

    def register_async_task(self) -> None:
        @repeat_every(seconds=5)
        def check_process() -> None:
            self.register_base_scheduler.check_process()

        @repeat_every(seconds=8)
        def check_scheduler_run() -> None:
            self.register_base_scheduler.run()

        @repeat_every(seconds=10)
        def check_redis_jobstores() -> None:
            self.register_base_scheduler.check_redis_jobstores()

        def scheduler_start():
            self.scheduler.start()

        def scheduler_shutdown():
            self.scheduler.shutdown()

        self.app.on_event("startup")(check_process)
        self.app.on_event("startup")(check_scheduler_run)
        self.app.on_event("startup")(check_redis_jobstores)
        self.app.on_event("startup")(scheduler_start)
        self.app.on_event("shutdown")(scheduler_shutdown)

    def add_job(self, job):
        if self.redis.hexists(self.jobs_key, job.id):
            raise ConflictingIdError(job.id)
        self.redis.hset(self.jobs_key, job.id, pickle.dumps(job.__getstate__(), self.pickle_protocol))
        if job.next_run_time:
            self.redis.zadd(self.run_times_key, {job.id: datetime_to_utc_timestamp(job.next_run_time)})

    def update_job(self, job):
        if not self.redis.hexists(self.jobs_key, job.id):
            raise JobLookupError(job.id)
        self.redis.hset(self.jobs_key, job.id, pickle.dumps(job.__getstate__(), self.pickle_protocol))
        if job.next_run_time:
            self.redis.zadd(self.run_times_key,
                            {job.id: datetime_to_utc_timestamp(job.next_run_time)})
        else:
            self.redis.zrem(self.run_times_key, job.id)

    def remove_job(self, job_id):
        if not self.redis.hexists(self.jobs_key, job_id):
            raise JobLookupError(job_id)
        self.redis.hdel(self.jobs_key, job_id)
        self.redis.zrem(self.run_times_key, job_id)

    def remove_all_jobs(self):
        self.redis.delete(self.jobs_key)
        self.redis.delete(self.run_times_key)

    def register_scheduler(self) -> None:
        redis_job_store = getattr(self.app, "redis_job_store", None)
        if redis_job_store is None:
            redis_job_store = RedisJobStore(
                host=self.redis_host,
                port=self.redis_port,
                username=self.username,
                password=self.password,
                jobs_key=self.jobs_key,
                run_times_key=self.run_times_key,
                ssl=self.ssl,
            )
            redis_job_store.add_job = self.add_job
            redis_job_store.update_job = self.update_job
            redis_job_store.remove_job = self.remove_job
            redis_job_store.remove_all_jobs = self.remove_all_jobs

        self.redis_job_store = redis_job_store
        setattr(self.app, "redis_job_store", self.redis_job_store)
        scheduler = getattr(self.app, "scheduler", None)
        if scheduler is None:
            scheduler = BackgroundScheduler()
            scheduler.configure(
                jobstores={
                    "default": self.redis_job_store
                },
                executors={
                    "default": ThreadPoolExecutor(
                        max_workers=self.thread_pool_size,
                    )
                },
                job_defaults={
                    "coalesce": self.job_coalesce,
                    "max_instance": self.job_max_instance,
                    "misfire_grace_time": self.job_misfire_grace_time,
                }
            )
        elif isinstance(scheduler, BackgroundScheduler):
            pass
        else:
            raise SchedulerError("FastAPI应用已经包含scheduler对象，但是该对象并非BackgroundScheduler")
        self.scheduler = scheduler
        setattr(self.app, "scheduler", self.scheduler)

    def add_task(
            self,
            func: Callable,
            job_id: str,
            trigger: str,
            crawler_info: Dict = None,
            **trigger_args
    ) -> None:
        '''
        :param trigger: interval 、 date or cron
        '''
        task_info = dict()
        task_info['func'] = obj_to_ref(func)
        task_info['job_id'] = job_id
        task_info['trigger'] = trigger
        task_info['crawler_info'] = crawler_info
        task_info['trigger_args'] = trigger_args
        self.register_base_scheduler.insert_task(task_info=task_info)

    def delete_task(
            self,
            job_id: str,
    ) -> None:
        self.register_base_scheduler.delete_task(job_id=job_id)

    def get_all_apscheduler_stores_jobs(self):
        redis_apscheduler_jobs_dict = dict()
        for apscheduler_stores_job_key in self.register_redis_db.get_stores_job_task():
            value_dict = dict()
            job_states_byte_dict = self.redis_job_store.redis.hgetall(apscheduler_stores_job_key)
            for job_states_key_byte, job_states_value_byte in job_states_byte_dict.items():
                job_states_key = job_states_key_byte.decode()
                job_states_value = pickle.loads(job_states_value_byte)
                # print(f"handle_before： {job_states_value}")
                if 'trigger' in job_states_value:
                    # todo: combining 情况下有问题
                    trigger = job_states_value['trigger'].__getstate__()
                    job_states_value['trigger'] = handle_trigger(trigger)
                if 'next_run_time' in job_states_value:
                    job_states_value['next_run_time'] = handle_datetime(agefield=job_states_value.get('next_run_time'))
                # print(f"handle_after： {job_states_value}")
                value_dict[job_states_key] = job_states_value
            redis_apscheduler_jobs_dict[apscheduler_stores_job_key] = value_dict
        return redis_apscheduler_jobs_dict

    def get_all_redis_tasks(self):
        all_tasks = dict()
        for all_task_key in self.register_redis_db.connection.keys(pattern=f'{self.project_name}:all:*'):
            value = self.register_redis_db.from_key_get_value(all_task_key)
            all_tasks[all_task_key] = value
        return all_tasks

    def get_all_redis_apscheduler_run_times(self):
        redis_apscheduler_run_time_dict = dict()
        for apscheduler_run_time_key in self.register_redis_db.get_stores_job_run_time_task():
            value_dict = dict()
            apscheduler_run_time_byte_list = self.redis_job_store.redis.zrange(
                apscheduler_run_time_key, 0, 4821384687, withscores=True)
            for job_states_key_byte, run_time__byte_timestamp in apscheduler_run_time_byte_list:
                job_states_key = job_states_key_byte.decode()
                value_dict[job_states_key] = run_time__byte_timestamp
            redis_apscheduler_run_time_dict[apscheduler_run_time_key] = value_dict
        return redis_apscheduler_run_time_dict

    def show_all_redis_key(self):
        return self.register_redis_db.connection.keys(pattern=f'{self.project_name}:*')

    def show_all_redis_nodes(self):
        node_dict = dict()
        for node_key in self.register_redis_db.connection.keys(pattern=f'{self.project_name}:node:*'):
            value = self.register_redis_db.connection.get(node_key)
            node_dict[node_key] = value
        return node_dict

    def show_all_redis_tasks(self):
        return self.get_all_redis_tasks()

    def show_all_scheduler_get_jobs(self):
        all_tasks = dict()
        for all_task_key in self.register_redis_db.connection.keys(pattern=f'{self.project_name}:running_job:*'):
            value = dict()
            value['approximate_start_time'] = self.register_redis_db.connection.get(all_task_key)
            value['data_ttl'] = self.register_redis_db.connection.ttl(all_task_key)
            all_tasks[all_task_key] = value
        return all_tasks

    def show_all_apscheduler_stores_jobs(self):
        return self.get_all_apscheduler_stores_jobs()

    def show_all_redis_apscheduler_run_times(self):
        return self.get_all_redis_apscheduler_run_times()

    def show_all_task_not_in_stores_jobs(self):
        problem_task = {}
        redis_apscheduler_job_id_list = []
        for redis_apscheduler_jobs_value in self.get_all_apscheduler_stores_jobs().values():
            redis_apscheduler_job_id_list.extend(redis_apscheduler_jobs_value.keys())
        for all_key, all_value in self.get_all_redis_tasks().items():
            if all_value.get('job_id') not in redis_apscheduler_job_id_list:
                problem_task[all_key] = all_value
        return problem_task

    def show_all_task_not_in_stores_run_times(self):
        problem_task = {}
        redis_apscheduler_run_time_job_id_list = []
        for redis_apscheduler_job_run_time_value in self.get_all_redis_apscheduler_run_times().values():
            redis_apscheduler_run_time_job_id_list.extend(redis_apscheduler_job_run_time_value.keys())
        for all_key, all_value in self.get_all_redis_tasks().items():
            if all_value.get('job_id') not in redis_apscheduler_run_time_job_id_list:
                problem_task[all_key] = all_value
        return problem_task
